@@ -22,7 +22,7 @@
  * Desc: Rtk fig functions
  * Author: Andrew Howard
  * Contributors: Richard Vaughan
- * CVS: $Id: rtk_fig.c,v 1.2 2006/02/23 00:29:55 gerkey Exp $
+ * CVS: $Id: rtk_fig.c 7942 2009-07-02 23:50:06Z gbiggs $
  *
  * Notes:
  *   Some of this is a horrible hack, particular the xfig stuff.
@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <float.h>
 #include "rtk.h"
 #include "rtkprivate.h"
 
@@ -508,10 +509,10 @@ void rtk_fig_calc(rtk_fig_t *fig)
   rtk_region_set_empty(fig->region);
 
   // Reset the figure bounding box.
-  fig->min_x = +DBL_MAX/2;
-  fig->min_y = +DBL_MAX/2;  
-  fig->max_x = -DBL_MAX/2;
-  fig->max_y = -DBL_MAX/2;
+  fig->min_x = +DBL_MAX / 2.0;
+  fig->min_y = +DBL_MAX / 2.0;
+  fig->max_x = -DBL_MAX / 2.0;
+  fig->max_y = -DBL_MAX / 2.0;
 
   // Recalculate strokes
   for (i = 0; i < fig->stroke_count; i++)
