@@ -23,7 +23,7 @@
 // Desc: Driver for detecting combined laser reflectors with B&W barcodes
 // Author: Andrew Howard
 // Date: 9 Jan 2004
-// CVS: $Id: laservisualbw.cc,v 1.14.2.2 2007/03/01 21:33:16 gerkey Exp $
+// CVS: $Id: laservisualbw.cc 7305 2009-01-27 01:18:55Z gbiggs $
 //
 // Theory of operation:
 //   Parses a laser scan to find the retro-reflective patches (lines or
@@ -40,7 +40,7 @@
 /** @{ */
 /** @defgroup driver_laservisualbw laservisualbw
  * @brief Black & white laser/visual barcode detector
- 
+
 @todo This driver has not been tested with the player 2 API.
 
 Parses a laser scan to find the retro-reflective patches (lines or
@@ -65,7 +65,7 @@ circular patches.
 
 @par Configuration requests
 
-- PLAYER_FIDUCIAL_GET_GEOM
+- PLAYER_FIDUCIAL_REQ_GET_GEOM
 
 @par Configuration file options
 
@@ -117,8 +117,6 @@ driver
 #include <string.h>
 #include <math.h>
 #include <stdlib.h>       // for atoi(3)
-#include <netinet/in.h>   // for htons(3)
-#include <unistd.h>
 
 #include <libplayercore/playercore.h>
 
@@ -156,8 +154,8 @@ class LaserVisualBW : public Driver
     double id_time;
   };
 
-  // Process incoming messages from clients 
-  int ProcessMessage (MessageQueue * resp_queue, player_msghdr * hdr, void * data);
+  // Process incoming messages from clients
+  int ProcessMessage (QueuePointer &resp_queue, player_msghdr * hdr, void * data);
 
   // Process laser data.
   // Returns non-zero if the laser data has been updated.
@@ -165,7 +163,7 @@ class LaserVisualBW : public Driver
 
   // Analyze the laser data to find fidicuials (reflectors).
   private: void FindLaserFiducials(double time, player_laser_data_t *data);
-    
+
   // Find the line of best fit for the given segment of the laser
   // scan.  Fills in the pose of the reflector relative to the laser.
   private: void FitLaserFiducial(player_laser_data_t *data, int first, int last, double pose[3]);
@@ -189,18 +187,18 @@ class LaserVisualBW : public Driver
   // Process any new camera data.
   private: int UpdateCamera(player_camera_data_t * data, double timestamp);
 
-  // Extract a bit string from the image.  
+  // Extract a bit string from the image.
   private: int ExtractSymbols(int x, unsigned int symbol_max_count, int symbols[]);
 
   // Extract a code from a symbol string.
   private: int ExtractCode(int symbol_count, int symbols[]);
-  
+
   // Write the device data (the data going back to the client).
   private: void WriteData();
 
   // Image processing
   private: double edge_thresh;
-  
+
   // Barcode tolerances
   private: int barcount;
   private: double barwidth;
@@ -242,9 +240,10 @@ class LaserVisualBW : public Driver
 
   // Dimensions of the zoomed image for the target fiducial (m).
   private: double zoomwidth, zoomheight;
-  
+
   // Local copy of the current fiducial data.
   private: player_fiducial_data_t fdata;
+  int fdata_allocated;
 };
 
 
@@ -256,7 +255,7 @@ Driver* LaserVisualBW_Init( ConfigFile* cf, int section)
 
 
 // a driver registration function
-void LaserVisualBW_Register(DriverTable* table)
+void laservisualbw_Register(DriverTable* table)
 {
   table->AddDriver("laservisualbw", LaserVisualBW_Init);
 }
@@ -271,7 +270,7 @@ LaserVisualBW::LaserVisualBW( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->laser_id, section, "requires",
                        PLAYER_LASER_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->laser = NULL;
@@ -281,7 +280,7 @@ LaserVisualBW::LaserVisualBW( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->ptz_id, section, "requires",
                        PLAYER_PTZ_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->ptz = NULL;
@@ -291,7 +290,7 @@ LaserVisualBW::LaserVisualBW( ConfigFile* cf, int section)
   if (cf->ReadDeviceAddr(&this->camera_id, section, "requires",
                        PLAYER_CAMERA_CODE, -1, NULL) != 0)
   {
-    this->SetError(-1);    
+    this->SetError(-1);
     return;
   }
   this->camera = NULL;
@@ -303,7 +302,7 @@ LaserVisualBW::LaserVisualBW( ConfigFile* cf, int section)
 
   // Image processing
   this->edge_thresh = cf->ReadFloat(section, "edge_thresh", 20);
-  
+
   // Default fiducial properties.
   this->barwidth = cf->ReadLength(section, "bit_width", 0.08);
   this->barcount = cf->ReadInt(section, "bit_count", 3);
@@ -330,6 +329,9 @@ LaserVisualBW::LaserVisualBW( ConfigFile* cf, int section)
 // Set up the device (called by server thread).
 int LaserVisualBW::Setup()
 {
+  fdata_allocated = 0;
+  fdata.fiducials = NULL;
+
   // Subscribe to the laser.
   if (!(laser = deviceTable->GetDevice (laser_id)))
   {
@@ -365,7 +367,7 @@ int LaserVisualBW::Setup()
     PLAYER_ERROR ("unable to subscribe to camera device");
     return -1;
   }
-  
+
   return 0;
 }
 
@@ -379,20 +381,20 @@ int LaserVisualBW::Shutdown()
   ptz->Unsubscribe(InQueue);
   camera->Unsubscribe(InQueue);
 
+  free(fdata.fiducials);
+
   return 0;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 // Process an incoming message
-int LaserVisualBW::ProcessMessage (MessageQueue * resp_queue, player_msghdr * hdr, void * data)
+int LaserVisualBW::ProcessMessage (QueuePointer &resp_queue, player_msghdr * hdr, void * data)
 {
   assert(hdr);
-  assert(data);
-  
+
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_LASER_DATA_SCAN, laser_id))
   {
-    assert(hdr->size == sizeof(player_laser_data_t));
     player_laser_data_t * l_data = reinterpret_cast<player_laser_data_t * > (data);
 
     UpdateLaser(l_data, hdr->timestamp);
@@ -402,45 +404,16 @@ int LaserVisualBW::ProcessMessage (MessageQueue * resp_queue, player_msghdr * hd
 
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_PTZ_DATA_STATE, ptz_id))
   {
-    assert(hdr->size == sizeof(player_ptz_data_t));
     UpdatePtz(reinterpret_cast<player_ptz_data_t * > (data), hdr->timestamp);
     return 0;
   }
 
   if(Message::MatchMessage (hdr, PLAYER_MSGTYPE_DATA, PLAYER_CAMERA_DATA_STATE, camera_id))
   {
-    assert(hdr->size == sizeof(player_camera_data_t));
     UpdateCamera(reinterpret_cast<player_camera_data_t * > (data), hdr->timestamp);
     return 0;
   }
 
-/*
- 
-  if (MatchMessage(hdr, PLAYER_MSGTYPE_REQ, PLAYER_FIDUCIAL_GET_GEOM, device_id))
-  {
-    assert(*resp_len>sizeof(player_fiducial_geom_t));
-    assert(*resp_len>sizeof(player_laser_geom_t));
-
-    int ret = laser->ProcessMessage( PLAYER_MSGTYPE_REQ, PLAYER_LASER_GET_GEOM, 
-           laser_id, 0, resp_data, resp_data, resp_len);
-    if (ret != PLAYER_MSGTYPE_RESP_ACK)
-    	return ret;
-  	assert(*resp_len == sizeof(player_laser_geom_t));
-  	player_laser_geom_t lgeom = * reinterpret_cast<player_laser_geom_t * > (resp_data);
-  	player_fiducial_geom_t * fgeom = reinterpret_cast<player_fiducial_geom_t * > (resp_data);
-
-    fgeom->pose[0] = lgeom.pose[0];
-    fgeom->pose[1] = lgeom.pose[1];
-    fgeom->pose[2] = lgeom.pose[2];
-    fgeom->size[0] = lgeom.size[0];
-    fgeom->size[1] = lgeom.size[1];
-    fgeom->fiducial_size[0] = ntohs((int) (this->barwidth * 1000));
-    fgeom->fiducial_size[1] = ntohs((int) (this->barwidth * 1000));
-  
-  	*resp_len=sizeof(player_fiducial_geom_t);
-  
-    return ret;
-  }*/
   return -1;
 }
 
@@ -449,7 +422,7 @@ int LaserVisualBW::ProcessMessage (MessageQueue * resp_queue, player_msghdr * hd
 int LaserVisualBW::UpdateLaser(player_laser_data_t * data, double timestamp)
 {
   this->laser_time = timestamp;
-  
+
   // Find possible fiducials in this scan.
   this->FindLaserFiducials(timestamp, data);
 
@@ -473,14 +446,14 @@ void LaserVisualBW::FindLaserFiducials(double time, player_laser_data_t *data)
 
   // Empty the fiducial list.
   this->fdata.fiducials_count = 0;
-  
+
   // Initialise patch statistics.
   mn = 0.0;
   mr = 0.0;
   mb = 0.0;
   mrr = 0.0;
   mbb = 0.0;
-    
+
   // Look for a candidate patch in scan.
   for (i = 0; i < data->ranges_count; i++)
   {
@@ -514,7 +487,7 @@ void LaserVisualBW::FindLaserFiducials(double time, player_laser_data_t *data)
       db = atan2(this->barwidth / 2, mr);
       valid &= (mrr < (dr * dr));
       valid &= (mbb < (db * db));
-      
+
       if (valid)
       {
         // Do a best fit to determine the pose of the reflector.
@@ -523,7 +496,7 @@ void LaserVisualBW::FindLaserFiducials(double time, player_laser_data_t *data)
         // Match this fiducial against the ones we are already tracking.
         this->MatchLaserFiducial(time, pose);
       }
-      
+
       mn = 0.0;
       mr = 0.0;
       mb = 0.0;
@@ -581,7 +554,7 @@ void LaserVisualBW::MatchLaserFiducial(double time, double pose[3])
   double mindr;
   fiducial_t *fiducial;
   fiducial_t *minfiducial;
-  
+
   // Observations must be at least this close to the existing
   // fiducial.
   mindr = this->max_dist;
@@ -606,18 +579,21 @@ void LaserVisualBW::MatchLaserFiducial(double time, double pose[3])
   // If we didnt find a matching fiducial, add a new one.
   if (minfiducial == NULL)
   {
-    if (this->fiducial_count < ARRAYSIZE(this->fiducials))
+    this->fiducial_count++;
+    if (this->fiducial_count+1 > this->fdata_allocated)
     {
-      minfiducial = this->fiducials + this->fiducial_count++;
-      minfiducial->id = -1;
-      minfiducial->pose[0] = pose[0];
-      minfiducial->pose[1] = pose[1];
-      minfiducial->pose[2] = pose[2];
-      minfiducial->laser_time = time;
-      minfiducial->ptz_select_time = -1;
-      minfiducial->ptz_lockon_time = -1;
-      minfiducial->id_time = -1;
+      this->fdata_allocated = this->fiducial_count+1;
+      this->fdata.fiducials = (player_fiducial_item_t*)realloc(this->fdata.fiducials, sizeof(this->fdata.fiducials[0])*this->fdata_allocated);
     }
+    minfiducial = &this->fiducials[this->fiducial_count-1];
+    minfiducial->id = -1;
+    minfiducial->pose[0] = pose[0];
+    minfiducial->pose[1] = pose[1];
+    minfiducial->pose[2] = pose[2];
+    minfiducial->laser_time = time;
+    minfiducial->ptz_select_time = -1;
+    minfiducial->ptz_lockon_time = -1;
+    minfiducial->id_time = -1;
   }
 
   // Otherwise, update the existing fiducial.
@@ -628,7 +604,7 @@ void LaserVisualBW::MatchLaserFiducial(double time, double pose[3])
     minfiducial->pose[2] = pose[2];
     minfiducial->laser_time = time;
   }
-  
+
   return;
 }
 
@@ -639,7 +615,7 @@ void LaserVisualBW::RetireLaserFiducials(double time, player_laser_data_t *data)
 {
   int i;
   fiducial_t *fiducial;
-  
+
   // Remove any old fiducials.
   for (i = 0; i < this->fiducial_count; i++)
   {
@@ -663,7 +639,7 @@ void LaserVisualBW::RetireLaserFiducials(double time, player_laser_data_t *data)
 int LaserVisualBW::UpdatePtz(player_ptz_data_t * data, double timestamp)
 {
   this->ptz_time = timestamp;
-  
+
   // Pick a fiducial to look at.
   this->SelectPtzTarget(timestamp, data);
 
@@ -694,7 +670,7 @@ void LaserVisualBW::SelectPtzTarget(double time, player_ptz_data_t *data)
   // Find one we havent looked at for while.
   this->ptz_fiducial = NULL;
   maxt = -1;
-  
+
   for (i = 0; i < this->fiducial_count; i++)
   {
     fiducial = this->fiducials + i;
@@ -712,7 +688,7 @@ void LaserVisualBW::SelectPtzTarget(double time, player_ptz_data_t *data)
     this->ptz_fiducial->ptz_select_time = time;
     this->ptz_fiducial->ptz_lockon_time = -1;
   }
-  
+
   return;
 }
 
@@ -733,7 +709,7 @@ void LaserVisualBW::ServoPtz(double time, player_ptz_data_t *data)
   // Deadband values.
   deadpan = 2;
   deadzoom = 2;
-  
+
   fiducial = this->ptz_fiducial;
   if (fiducial == NULL)
   {
@@ -765,12 +741,12 @@ void LaserVisualBW::ServoPtz(double time, player_ptz_data_t *data)
       tilt = maxtilt * sin((time - fiducial->ptz_lockon_time) /
                            this->max_ptz_attention * 2 * M_PI);
   }
-  
+
   // Compose the command packet to send to the PTZ device.
-  cmd.pan = pan;
-  cmd.tilt = tilt;
-  cmd.zoom = zoom;
-  
+  cmd.pan = static_cast<float> (pan);
+  cmd.tilt = static_cast<float> (tilt);
+  cmd.zoom = static_cast<float> (zoom);
+
   this->ptz->PutMsg(InQueue, PLAYER_MSGTYPE_CMD, PLAYER_PTZ_CMD_STATE, &cmd, sizeof(cmd), NULL);
 
   // Compute the dimensions of the image at the range of the target fiducial.
@@ -791,9 +767,9 @@ int LaserVisualBW::UpdateCamera(player_camera_data_t * data, double timestamp)
   int symbols[480];
 
   this->camera_time = timestamp;
-  
+
   best_id = -1;
-  
+
   // Barcode may not be centered, so look across entire image
   for (x = 0; x < this->camera_data.width; x += 16)
   {
@@ -828,7 +804,7 @@ int LaserVisualBW::UpdateCamera(player_camera_data_t * data, double timestamp)
       this->ptz_fiducial->id_time = timestamp;
     }
   }
-  
+
   return 1;
 }
 
@@ -889,7 +865,7 @@ int LaserVisualBW::ExtractSymbols(int x, unsigned int symbol_max_count, int symb
       fn += fabs(kernel[j + 2]);
     }
     fv /= fn;
-    
+
     // Pick the transitions
     if (state == -1)
     {
@@ -966,7 +942,7 @@ int LaserVisualBW::ExtractCode(int symbol_count, int symbols[])
     };
 
   best_digit = -1;
-  
+
   // Note that each code has seven symbols in it, not counting the
   // initial space.
   for (i = 0; i < symbol_count - 7; i++)
@@ -998,12 +974,12 @@ int LaserVisualBW::ExtractCode(int symbol_count, int symbols[])
 
       best_err = this->err_first;
       best_digit = -1;
-      
+
       // Read the code digit (4 symbols) and compare against the known
       // digit patterns
       for (k = 0; k < (int) (sizeof(digits) / sizeof(digits[0])); k++)
       {
-        err[k] = 0;        
+        err[k] = 0;
         for (j = 0; j < 4; j++)
         {
           wm = digits[k][j];
@@ -1036,7 +1012,7 @@ int LaserVisualBW::ExtractCode(int symbol_count, int symbols[])
       // Stop if we found a valid digit
       if (best_digit >= 0)
         break;
-    }    
+    }
   }
 
   //if (best_digit >= 0)
@@ -1065,7 +1041,7 @@ void LaserVisualBW::WriteData()
     // scan.
     if (fiducial->laser_time != this->laser_time)
       continue;
-    
+
     r = sqrt(fiducial->pose[0] * fiducial->pose[0] +
              fiducial->pose[1] * fiducial->pose[1]);
     b = atan2(fiducial->pose[1], fiducial->pose[0]);
@@ -1077,12 +1053,12 @@ void LaserVisualBW::WriteData()
     data.fiducials[data.fiducials_count].pose.pyaw = o;
     data.fiducials_count++;
   }
-  
+
   // Compute the data timestamp (from laser).
   timestamp = this->laser_time;
-  
+
   // Copy data to server.
-  Publish(device_addr, NULL, PLAYER_MSGTYPE_DATA, PLAYER_FIDUCIAL_DATA_SCAN, (void*) &data, sizeof(data), &timestamp);
+  Publish(device_addr, PLAYER_MSGTYPE_DATA, PLAYER_FIDUCIAL_DATA_SCAN, (void*) &data, sizeof(data), &timestamp);
 }
 
 
