@@ -1,6 +1,28 @@
+/*
+ *  Player - One Hell of a Robot Server
+ *  Copyright (C) 2006
+ *     Kathy Fung, Toby Collett
+ *
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+
 #include "nav200.h"
 
 
+
 Nav200::Nav200()
 {
 }
@@ -10,63 +32,25 @@ Nav200::~Nav200()
 {
 }
 
-int Nav200::Initialise(const char * port, int rate)
+int Nav200::Initialise(Driver* sn2002, Device* opaque2, player_devaddr_t opaque_id2 )
 {
+  this->sn200 = sn2002;
+  this->opaque_id = opaque_id2;
+  this->opaque = opaque2;
   bytesReceived = 0;
-  fd = -1;
 
-  // open the serial port
-  fd = open(port, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK);
-  if ( fd<0 )
-  {
-    fprintf(stderr, "Could not open serial device %s\n",port);
-    return -1;
-  }
-
-  // save the current io settings
-  tcgetattr(fd, &oldtio);
-
-  // set up new settings
-  struct termios newtio;
-  memset(&newtio, 0,sizeof(newtio));
-  newtio.c_cflag = CS8 | CREAD | PARENB;
-  newtio.c_iflag = INPCK;
-  newtio.c_oflag = 0;
-  newtio.c_lflag = 0;
-
-  // activate new settings
-  tcflush(fd, TCIFLUSH);
-  if (cfsetispeed(&newtio, rate) < 0 || cfsetospeed(&newtio, rate) < 0)
-  {
-    fprintf(stderr,"Failed to set serial baud rate: %d\n", rate);
-    tcsetattr(fd, TCSANOW, &oldtio);
-    close(fd);
-    fd = -1;
-    return -1;
-  }
-  tcsetattr(fd, TCSANOW, &newtio);
-  tcflush(fd, TCIOFLUSH);
-
-  // clear the input buffer in case junk data is on the port
-  usleep(10000);
-  tcflush(fd, TCIFLUSH);
   return 0;
 }
 
 int Nav200::Terminate()
 {
-  // restore old port settings
-  if (fd > 0)
-  {
-    tcsetattr(fd, TCSANOW, &oldtio);
-    close(fd);
-  }
   return 0;
 }
 
 /*****************
 * Standby mode
 *******************/
+
 bool Nav200::EnterStandby()
 {
   WriteCommand('S','A',0,NULL);
@@ -588,12 +572,11 @@ void Nav200::PrintErrorMsg(void)
 // Write a packet to Nav200
 int Nav200::WriteCommand(char mode, char function, int dataLength, uint8_t * data)
 {
-  if (fd < 0)
-    return -1;
+/*  if (fd < 0)
+    return -1;*/
 
-  uint8_t * buffer;
   int length = dataLength+5;
-  buffer = new uint8_t[length];
+  uint8_t * buffer = new uint8_t[length];
 
   // Create header
   buffer[0] = STX;
@@ -607,35 +590,66 @@ int Nav200::WriteCommand(char mode, char function, int dataLength, uint8_t * dat
   // Create footer (CRC)
   buffer[length-1] = CreateCRC(buffer, 4 + dataLength);
 
-  // Make sure both input and output queues are empty
-  tcflush(fd, TCIOFLUSH);
+/*  // Make sure both input and output queues are empty
+  tcflush(fd, TCIOFLUSH);*/
 
-  // switch to blocking IO for the write
+/*  // switch to blocking IO for the write
   int flags = fcntl(fd, F_GETFL);
   if (flags < 0 || fcntl(fd,F_SETFL,flags &~O_NONBLOCK) < 0)
   {
     fprintf(stderr,"Error changing to blocking write (%d - %s), disabling\n",errno,strerror(errno));
     tcsetattr(fd, TCSANOW, &oldtio);
     fd = -1;
+    delete [] buffer;
     return -1;
   }
+*/
+  player_opaque_data_t mData;
+  mData.data_count = length;
+  mData.data = buffer;
 
-  if((length && (write(fd, buffer, length)) < length))
+  // before we send a command to the NAV200, flush and data coming in from the remote file
+  sn200->InQueue->SetFilter(opaque_id.host, opaque_id.robot, opaque_id.interf, opaque_id.index, PLAYER_MSGTYPE_DATA, PLAYER_OPAQUE_DATA_STATE);
+  
+  //puts("waiting for data");
+  sn200->ProcessMessages();
+  
+  // before we send a command to the NAV200, flush and data coming in from the remote file
+  if (bytesReceived)
+  {
+
+    do
+    {
+        PLAYER_WARN1("Buffer was not empty on NAV200 (%d), flushing",bytesReceived);
+        bytesReceived = 0;
+    	usleep(100000);
+    	sn200->ProcessMessages();
+    } while (bytesReceived != 0);
+  }
+  sn200->InQueue->ClearFilter();
+  
+  opaque->PutMsg(sn200->InQueue, PLAYER_MSGTYPE_CMD, PLAYER_OPAQUE_CMD_DATA, reinterpret_cast<void*>(&mData), 0, NULL);
+  
+/*  if((length && (write(fd, buffer, length)) < length))
   {
     fprintf(stderr,"Error writing to FOB (%d - %s), disabling\n",errno,strerror(errno));
     tcsetattr(fd, TCSANOW, &oldtio);
     fd = -1;
+    delete [] buffer;
     return -1;
-  }
+  }*/
 
-  // restore flags
+/*  // restore flags
   if (fcntl(fd,F_SETFL,flags) < 0)
   {
     fprintf(stderr,"Error restoring file mode (%d - %s), disabling\n",errno,strerror(errno));
     tcsetattr(fd, TCSANOW, &oldtio);
     fd = -1;
+    delete [] buffer;
     return -1;
-  }
+  }*/
+
+  delete [] buffer;
 
   return 0;
 }
@@ -645,41 +659,26 @@ int Nav200::WriteCommand(char mode, char function, int dataLength, uint8_t * dat
 // Read a packet from the Nav200
 int Nav200::ReadFromNav200(int timeout_usec)
 {
-  int ret;
+  
   int dataLength = 0;
 
   struct timeval start, now;
   gettimeofday(&start,NULL);
   
+  sn200->InQueue->SetFilter(opaque_id.host, opaque_id.robot, opaque_id.interf, opaque_id.index, PLAYER_MSGTYPE_DATA, PLAYER_OPAQUE_DATA_STATE);
+  
   for (;;)
   {
-    gettimeofday(&now,NULL);
-    if ((now.tv_sec - start.tv_sec) * 1e6 + now.tv_usec - start.tv_usec > timeout_usec)
-    {
-      fprintf(stderr,"Timed out waiting for packet %d uSecs passed\n",static_cast<int>((now.tv_sec - start.tv_sec) * 1e6 + now.tv_usec - start.tv_usec));
-      return -1;
-    }
-
-    ret = read(fd, &receivedBuffer[bytesReceived], BUFFER_SIZE - bytesReceived);
-    if (ret < 0)
-    {
-      fprintf(stderr,"Got error while reading %d %s\n",errno, strerror(errno));
-      return ret;
-    }
-  
-    bytesReceived += ret;
-
-    if (ret == 0)
-    {
-      usleep(1000);
-      continue;
-    }
-  
+    //puts("waiting for data");
+    sn200->ProcessMessages();
+    
     // do we have enough for a header?
     while (bytesReceived > 4)
     {
+      //PLAYER_MSG4(2, "recieved STX %d data len %d mode %c fun %c", receivedBuffer[0], receivedBuffer[1], receivedBuffer[2], receivedBuffer[3]);
       if (receivedBuffer[0] != STX)
-      {
+      { 
+    	bool found = false;
         // bad thing, we dont have the correct start to a message
         for(int i=1; i<bytesReceived; i++)
         { // find where STX is
@@ -687,8 +686,15 @@ int Nav200::ReadFromNav200(int timeout_usec)
           { // move so STX is at start
             memmove(receivedBuffer, receivedBuffer+i, bytesReceived-i);
             bytesReceived-=i;
+            found = true;
+            break;
           }
         }
+        // If none of the data points are STX then all are (essentially) removed
+        if (!found)
+        {
+        	bytesReceived = 0;
+        }
         continue;
       }
   
@@ -706,45 +712,63 @@ int Nav200::ReadFromNav200(int timeout_usec)
       if (CreateCRC(receivedBuffer, dataLength)) // change that later!!
       {// bad thing, we dont have the correct start to a message
         fprintf(stderr,"bad CRC!!!\n");
+        bool found = false;
         for(int i=1; i<bytesReceived; i++)
         { // find where STX is
           if(receivedBuffer[i]==STX)
           { // move so STX is at start
             memmove(receivedBuffer, receivedBuffer+i, bytesReceived-i);
             bytesReceived-=i;
+            found = true;
+            break;
           }
         }
+        // If none of the data points are STX then all are (essentially) removed
+        if (!found)
+        {
+        	bytesReceived = 0;
+        }
         continue;
       }
       // if good CRC we actually have a packet so assemble our NAV200 message and return it to the application
       else
       {
-        if(receivedBuffer[3]=='E'){
-          error.F0 = receivedBuffer[4];
-          error.F1 = receivedBuffer[5];
-          error.F2 = receivedBuffer[6];
-          error.F3 = receivedBuffer[7];
-  
-          //check out what the error is and it out
-          PrintErrorMsg();
-  
-          return -2;
+          packet.header = receivedBuffer[0];
+          packet.length = receivedBuffer[1];
+          packet.mode = receivedBuffer[2];
+          packet.function = receivedBuffer[3];
+          packet.dataLength = packet.length-HEADER_SIZE-FOOTER_SIZE;
+          memcpy(packet.data, receivedBuffer+4, packet.dataLength);
+          packet.BCC = receivedBuffer[4+packet.dataLength];
+    
+          memmove(receivedBuffer, receivedBuffer+dataLength, bytesReceived-dataLength);
+          bytesReceived-=dataLength;
+          sn200->InQueue->ClearFilter();
+          
+          if(receivedBuffer[3]=='E'){
+	          error.F0 = receivedBuffer[4];
+	          error.F1 = receivedBuffer[5];
+	          error.F2 = receivedBuffer[6];
+	          error.F3 = receivedBuffer[7];
+	  
+	          //check out what the error is and it out
+	          PrintErrorMsg();
+	          return -2;
         }
-  
-        packet.header = receivedBuffer[0];
-        packet.length = receivedBuffer[1];
-        packet.mode = receivedBuffer[2];
-        packet.function = receivedBuffer[3];
-        packet.dataLength = packet.length-HEADER_SIZE-FOOTER_SIZE;
-        memcpy(packet.data, receivedBuffer+4, packet.dataLength);
-        packet.BCC = receivedBuffer[4+packet.dataLength];
-  
-        memmove(receivedBuffer, receivedBuffer+dataLength, bytesReceived-dataLength);
-        bytesReceived-=dataLength;
+
         return 1;
       }
     }
+    gettimeofday(&now,NULL);
+    if ((now.tv_sec - start.tv_sec) * 1e6 + now.tv_usec - start.tv_usec > timeout_usec)
+    {
+      fprintf(stderr,"Timed out waiting for packet %d uSecs passed\n",static_cast<int>((now.tv_sec - start.tv_sec) * 1e6 + now.tv_usec - start.tv_usec));
+      sn200->InQueue->ClearFilter();
+      return -1;
+    }
+    usleep(1000);
   }
+  sn200->InQueue->ClearFilter();
   return 0;
 }
 
@@ -752,7 +776,7 @@ int Nav200::ReadFromNav200(int timeout_usec)
 uint8_t Nav200::CreateCRC(uint8_t* data, ssize_t len)
 {
   uint8_t result = 0;
-  for (int ii = 0; ii < len; ++ii)
+  for (ssize_t ii = 0; ii < len; ++ii)
     result ^= data[ii];
   return result;
 }
