@@ -21,7 +21,7 @@
  * Desc: Public strutures, functions
  * Author: Andrew Howard
  * Date: 14 May 2002
- * CVS: $Id: playerv.h,v 1.44.4.2 2007/01/31 22:06:15 gerkey Exp $
+ * CVS: $Id: playerv.h 8062 2009-07-21 03:47:16Z gbiggs $
  ***************************************************************************/
 
 #ifndef PLAYERV_H
@@ -39,6 +39,7 @@
  ***************************************************************************/
 
 #define MAP_UPDATE_TIME 1.0
+#define VECTORMAP_UPDATE_TIME 1.0
 
 /***************************************************************************
  * Default colors
@@ -47,6 +48,8 @@
 #define COLOR_GRID_MAJOR         0xC0C0C0
 #define COLOR_GRID_MINOR         0xE0E0E0
 #define COLOR_AIO                0x000000
+#define COLOR_ACTARRAY_DATA      0x00C000
+#define COLOR_ACTARRAY_CMD       0x0000C0
 #define COLOR_DIO                0x000000
 #define COLOR_LASER              0x0000C0
 #define COLOR_LASER_EMP          0xD0D0FF
@@ -170,6 +173,47 @@ void create_proxy(device_t *device, opt_t *opt,
                   mainwnd_t *mainwnd, playerc_client_t *client);
 
 
+ /***************************************************************************
+  * ActArray device
+  ***************************************************************************/
+
+ // ActArray device info
+ typedef struct
+ {
+   // Driver name
+   char *drivername;
+
+   // Menu stuff
+   rtk_menu_t *menu;
+   rtk_menuitem_t *subscribe_item;
+   rtk_menuitem_t *command_item;
+
+   // Figures for drawing the actarray
+   rtk_fig_t **actuator_fig;
+   rtk_fig_t **actuator_fig_cmd;
+   double * lastvalue;
+   int fig_count;
+   mainwnd_t *mainwnd;
+
+   // ActArray device proxy
+   playerc_actarray_t *proxy;
+
+   // Timestamp on most recent data
+   double datatime;
+
+ } actarray_t;
+
+ // Create a actarray device
+ actarray_t *actarray_create(mainwnd_t *mainwnd, opt_t *opt, playerc_client_t *client,
+                       int index,  const char *drivername, int subscribe);
+
+ // Destroy a actarray device
+ void actarray_destroy(actarray_t *actarray);
+
+ // Update a actarray device
+ void actarray_update(actarray_t *actarray);
+
+
  /***************************************************************************
   * Bumper device
   ***************************************************************************/
@@ -185,7 +229,9 @@ void create_proxy(device_t *device, opt_t *opt,
    rtk_menuitem_t *subscribe_item;
 
    // Figures for drawing the bumper
-   rtk_fig_t *scan_fig[PLAYERC_BUMPER_MAX_SAMPLES];
+   rtk_fig_t **scan_fig;
+   int fig_count;
+   mainwnd_t *mainwnd;
 
    // Bumper device proxy
    playerc_bumper_t *proxy;
@@ -229,6 +275,7 @@ typedef struct
   rtk_menuitem_t *subscribe_item;
   rtk_menuitem_t *res025_item, *res050_item, *res100_item;
   rtk_menuitem_t *range_mm_item, *range_cm_item, *range_dm_item;
+  rtk_menuitem_t *style_item;
   // Figure for drawing the scan
   rtk_fig_t *scan_fig;
 
@@ -360,6 +407,8 @@ typedef struct
   rtk_menuitem_t *pose_mode_item;
   rtk_menuitem_t *enable_item, *disable_item;
   rtk_menuitem_t *frame_item;
+ rtk_menuitem_t *open_item;
+ rtk_menuitem_t *close_item;
 
   // Figures
   rtk_fig_t *grip_fig;
@@ -469,7 +518,9 @@ typedef struct
   rtk_menuitem_t *subscribe_item;
 
   // Figures for drawing the sonar scan
-  rtk_fig_t *scan_fig[PLAYERC_SONAR_MAX_SAMPLES];
+  rtk_fig_t **scan_fig;
+  int fig_count;
+  mainwnd_t *mainwnd;
 
   // Sonar device proxy
   playerc_sonar_t *proxy;
@@ -508,7 +559,9 @@ typedef struct
   rtk_menuitem_t *subscribe_item;
 
   // Figures for drawing the sonar scan
-  rtk_fig_t *scan_fig[PLAYERC_IR_MAX_SAMPLES];
+  rtk_fig_t **scan_fig;
+  int fig_count;
+  mainwnd_t *mainwnd;
 
   // Sonar device proxy
   playerc_ir_t *proxy;
@@ -530,6 +583,98 @@ void ir_destroy(ir_t *ir);
 void ir_update(ir_t *ir);
 
 
+/***************************************************************************
+ * Ranger
+ ***************************************************************************/
+
+// Ranger device info
+typedef struct
+{
+  // Driver name
+  char *drivername;
+
+  // Ranger device proxy
+  playerc_ranger_t *proxy;
+
+  // Timestamp on most recent data
+  double datatime;
+
+  // Menu stuff
+  rtk_menu_t *menu;
+  rtk_menuitem_t *subscribe_item;
+  rtk_menuitem_t *style_item;
+  rtk_menuitem_t *intns_item;
+  // Figure for drawing the scan
+  rtk_fig_t **scan_fig;
+  // Need to track this for creating figures
+  mainwnd_t *mainwnd;
+
+  // Properties from the device that may be necessary
+  double start_angle;
+  double angular_res;
+
+} ranger_t;
+
+
+// Create a ranger device
+ranger_t *ranger_create(mainwnd_t *mainwnd, opt_t *opt, playerc_client_t *client,
+                        int index,  const char *drivername, int subscribe);
+
+// Destroy a ranger device
+void ranger_destroy(ranger_t *ranger);
+
+// Update a ranger device
+void ranger_update(ranger_t *ranger);
+
+
+/***************************************************************************
+ * Camera device
+ ***************************************************************************/
+
+// Camera device info
+typedef struct
+{
+  // Driver name
+  char *drivername;
+
+  // Camera device proxy
+  playerc_camera_t *proxy;
+
+  // Menu stuff
+  rtk_menu_t *menu;
+  rtk_menuitem_t *subscribe_item;
+  rtk_menuitem_t *stats_item;
+  rtk_menuitem_t *scale_item;
+
+  // Figure for drawing the camera scan
+  rtk_fig_t *image_fig;
+  int image_init;
+
+  // Image scale (m/pixel)
+  double scale;
+
+  // Image buffer
+  int allocated_size;
+  unsigned char *img_buffer;
+
+  // Timestamp on most recent data
+  double datatime;
+
+} camera_t;
+
+
+// Create a camera device
+camera_t *camera_create(mainwnd_t *mainwnd, opt_t *opt, playerc_client_t *client,
+                                int index, const char *drivername, int subscribe);
+
+// Destroy a camera device
+void camera_destroy(camera_t *camera);
+
+// Update a camera device
+void camera_update(camera_t *camera);
+
+
+
 /***************************************************************************
  * Blobfinder device
  ***************************************************************************/
@@ -771,4 +916,41 @@ void map_destroy(map_t *map);
 void map_update(map_t *map);
 
 
+/***************************************************************************
+ * Vector Map device
+ ***************************************************************************/
+
+// Vector Map device info
+typedef struct
+{
+  // Driver name
+  char *drivername;
+
+  // Map device proxy
+  playerc_vectormap_t *proxy;
+
+  // Timestamp on most recent data
+  double datatime;
+
+  // Menu stuff
+  rtk_menu_t *menu;
+  rtk_menuitem_t *subscribe_item;
+  rtk_menuitem_t *continuous_item;
+  // Figures
+  rtk_fig_t *fig;
+
+} vectormap_t;
+
+
+// Create a map device
+vectormap_t *vectormap_create(mainwnd_t *mainwnd, opt_t *opt, playerc_client_t *client,
+                    int index,  const char *drivername, int subscribe);
+
+// Destroy a map device
+void vectormap_destroy(vectormap_t *map);
+
+// Update a map device
+void vectormap_update(vectormap_t *map);
+
+
 #endif
